# Nintendo Switch Gamepad Emulator

NSGamepad is gamepad software for the Nintendo Switch. It can be used to build
custom controllers or to create bots to automate boring parts of games. The
concept is not new and has been done with AVR 32u4, AVR 16u2, and Teensy with
the LUFA library. This project runs on SAMD processors and can be built using
the Arduino IDE.

NSGamepad not a hardware hack or mod. It is open source USB gamepad software.

NSGamepad is built on NicoHood's HID library. In fact, it is a customized
version of the library Gamepad code. It is also an example of how to use the
Arduino PluggableUSB/PluggableHID/NicoHood HID library to emulate a real world
USB device.

Adafruit SAMD boards support capacitive touch so it should be possible to build
soft game controllers using conductive thread and conductive fabric. Also
Adafruit has SAMD51 (M4) boards running as fast as 200 MHz (overclocked).
Maybe TensorFlow Lite can do voice/word recognition and trigger button presses.

## Build this code in safe workspace

The final step to activate the gamepad is to modify the USB Vendor ID (VID) and
Product ID (PID) to match an existing Nintendo Switch gamepad. I highly
recommend creating a safe Arduino IDE workspace for this project. If this is
not done, the VID and PID change for all projects.

See the following page for background on the Arduino Portable IDE.

https://www.arduino.cc/en/Guide/PortableIDE

On an Ubuntu Linux system, the following creates a directory with a safe
Arduino IDE workspace.

```
$ tar xf ~/Downloads/arduino-1.8.13-linux64.tar.xz
$ cd arduino-1.8.13
$ mkdir portable
$ ./arduino &
```

The portable directory hold sketches, libraries, and board files. These
directories are separate from the ~/Arduino and ~/.arduino15 directories use in
normal mode.

In the Arduino IDE, go to File | Preferences then uncheck "Check for updates on
startup". This is optional but if not done, board updates may wipe out changes
made for this project.

## Install SAMD board packages

Install the Arduino SAMD board package. Optionally, install the Adafruit SAMD
board package version 1.5.14. Versions newer than 1.5.14 do not work.  See
(adafruit.com)[https://learn.adafruit.com/adafruit-metro-m0-express-designed-for-circuitpython/arduino-ide-setup]
for details.

## Install libraries

Use Library manager to install the Bounce2 library which is used by the
NSGamepad example. You could use a different debounce library or write you own
debounce code.

Install the latest version of the NicoHood HID library from github.com. This
is currently the only way to get the latest version that works with SAMD boards.
Do not use Library Manager because that version does not have the fixes for SAMD
boards.

```
cd portable/sketchbook/libraries
git clone https://github.com/NicoHood/HID HID-Project
cd ../../..
```

## Merge source code from NSGaget_HID/src to HID-Project
Copy the four source code files in the src directory to directories with the
same names in HID-Project library installation directory.

```
NSGadget_HID$ ls -R src
src:
HID-APIs  SingleReport

src/HID-APIs:
NSGamepadAPI.h  NSGamepadAPI.hpp

src/SingleReport:
SingleNSGamepad.cpp  SingleNSGamepad.h
```

Add one line to HID-Project.h.
```
nano portable/sketchbook/libraries/HID-Project/src/HID-Project.h
```

Add this line following the line with "SingleGamepad.h"
```
#include "SingleReport/SingleNSGamepad.h"
```

## Patch SAMD board files

The next step changes the VID/PID. In this example, the board is an Arduino MKR
WiFi 1010 but any Arduino board with a SAMD21 processor should work including
the Nano 33 IoT and Zero. Use your favorite editor (or use nano) to change the
boards.txt file. The version number (1.8.4) will change so substitute the
correct one.

```
$ nano portable/packages/arduino/hardware/samd/1.8.4/boards.txt
```

Find the section for the board being used. In this case I am using a MKR WiFi
1010. See the following for the changes. The original values are commented out
so it is easy to switch back to normal mode. These values make the
Nintendo Switch think it is talking to a Hori HoriPAD game controller.
The changes will be erased if the Arduino SAMD board package is updated
or reinstalled. If after uploading a new sketch and the board stops working on
the Switch, make sure VID/PID change has not been lost.

```
# Arduino MKR WiFi 1010
# --------------------
mkrwifi1010.name=Arduino MKR WiFi 1010
.
.
.
mkrwifi1010.build.variant=mkrwifi1010
#mkrwifi1010.build.vid=0x2341
#mkrwifi1010.build.pid=0x8054
mkrwifi1010.build.vid=0x0f0d
mkrwifi1010.build.pid=0x00c1
mkrwifi1010.bootloader.tool=openocd
```

For Adafruit SAMD boards, the boards.txt is in a different location.

```
$ nano portable/packages/adafruit/hardware/samd/1.5.14/boards.txt
```

For Arduino Leonardo or Micro, the boards.txt is yet another location.

```
nano hardware/arduino/avr/boards.txt
```

## USB Descriptor Differences

These differences do not stop NSGamepad from working with the Switch. If
Nintendo changes the Switch firmware, NSGamepad may stop working so the
differences are listed here for future explorers.

### USB CDC ACM serial

The real gamepad does not have a USB CDC ACM port but NSGamepad does. The
serial port is also used to upload new sketches so removing it makes the board
harder to use. Remember new sketches always can be uploaded by double clicking the
board RESET button before starting the IDE upload.

To remove the CDC descriptors on Arduino SAMD boards, open the boards.txt file.

```
$ nano portable/packages/arduino/hardware/samd/1.8.4/boards.txt
```

Add "-DCDC_DISABLED" to the build.extra_flags as shown below.

```
# Arduino MKR WiFi 1010
# --------------------
mkrwifi1010.name=Arduino MKR WiFi 1010
.
.
.
mkrwifi1010.build.board=SAMD_MKRWIFI1010
mkrwifi1010.build.core=arduino
#mkrwifi1010.build.extra_flags=-DUSE_ARDUINO_MKR_PIN_LAYOUT -D__SAMD21G18A__ {build.usb_flags} -DUSE_BQ24195L_PMIC
mkrwifi1010.build.extra_flags=-DCDC_DISABLED -DUSE_ARDUINO_MKR_PIN_LAYOUT -D__SAMD21G18A__ {build.usb_flags} -DUSE_BQ24195L_PMIC
mkrwifi1010.build.ldscript=linker_scripts/gcc/flash_with_bootloader.ld
mkrwifi1010.build.openocdscript=openocd_scripts/arduino_zero.cfg
```

The above does not work for Adafruit SAMD boards. To remove CDC descriptors
comment out "#define CDC_ENABLED" in USBDesc.h.

```
nano portable/packages/adafruit/hardware/samd/1.5.9/cores/arduino/USB/USBDesc.h
```

```
// CDC or HID can be enabled together.
//#define CDC_ENABLED
#define PLUGGABLE_USB_ENABLED

#ifdef CDC_ENABLED
```

### USB Device Descriptors

The class, subclass, and protocol are different even after the CDC port is
removed. This can be fixed by changing source files in the board package.

For Arduino SAMD boards, bracket two lines of code in USBCore.cpp with #ifdef CDC_ENABLED/#endif.

```
nano portable/packages/arduino/hardware/samd/1.8.4/cores/arduino/USB/USBCore.cpp
```

```
	{
#ifdef CDC_ENABLED
		if (setup.wLength == 8)
			_cdcComposite = 1;
#endif

		desc_addr = _cdcComposite ?  (const uint8_t*)&USB_DeviceDescriptorB : (const uint8_t*)&USB_DeviceDescriptor;
```

For Adafruit SAMD boards, do the same thing but on a different version of the file.

```
nano portable/packages/adafruit/hardware/samd/1.5.9/cores/arduino/USB/USBCore.cpp
```

### OUT Endpoint

The real gamepad has an OUT as well as an IN endpoint. I am guessing the OUT
endpoint is intended for rumble control but since the controller does not have
rumble motors nothing is sent to the endpoint.

